from __future__ import absolute_import

__author__ = 'Josue Kouka'
__email__ = 'josuebrunel@gmail.com'
__version__ = "0.1.8"


from yahoo_oauth.yahoo_oauth import OAuth1, OAuth2
from yahoo_oauth.logger import YahooLogger
